@@ -1,6 +1,6 @@
-from collections import OrderedDict
 import sys
 import os
+import time
 import datetime as dt
 
 import alabaster
@@ -27,13 +27,18 @@ issues_github_path = "marshmallow-code/marshmallow"
 
 templates_path = ["_templates"]
 
+# Use SOURCE_DATE_EPOCH for reproducible build output https://reproducible-builds.org/docs/source-date-epoch/
+build_date = dt.datetime.utcfromtimestamp(
+    int(os.environ.get("SOURCE_DATE_EPOCH", time.time()))
+)
+
 source_suffix = ".rst"
 master_doc = "index"
 
 project = "marshmallow"
 copyright = (
     ' {:%Y} <a href="https://stevenloria.com">Steven Loria</a> and contributors'.format(
-        dt.datetime.utcfromtimestamp(os.path.getmtime("../CHANGELOG.rst"))
+        build_date
     )
 )
 
@@ -46,11 +51,13 @@ exclude_patterns = ["_build"]
 html_theme_path = [alabaster.get_path()]
 html_theme = "alabaster"
 html_static_path = ["_static"]
+html_css_files = ["css/versionwarning.css"]
 templates_path = ["_templates"]
 html_show_sourcelink = False
 
 html_theme_options = {
     "logo": "marshmallow-logo.png",
+    "logo_name": None,
     "description": "Object serialization and deserialization, lightweight and fluffy.",
     "description_font_style": "italic",
     "github_user": "marshmallow-code",
@@ -66,28 +73,23 @@ html_theme_options = {
     "warn_bg": "#FFC",
     "warn_border": "#EEE",
     # Used to populate the useful-links.html template
-    "extra_nav_links": OrderedDict(
-        [
-            ("marshmallow @ PyPI", "https://pypi.python.org/pypi/marshmallow"),
-            ("marshmallow @ GitHub", "https://github.com/marshmallow-code/marshmallow"),
-            ("Issue Tracker", "https://github.com/marshmallow-code/marshmallow/issues"),
-            (
-                "Ecosystem",
-                "https://github.com/marshmallow-code/marshmallow/wiki/Ecosystem",
-            ),
-        ]
-    ),
+    "extra_nav_links": {
+        "marshmallow @ PyPI": "https://pypi.python.org/pypi/marshmallow",
+        "marshmallow @ GitHub": "https://github.com/marshmallow-code/marshmallow",
+        "Issue Tracker": "https://github.com/marshmallow-code/marshmallow/issues",
+        "Ecosystem": "https://github.com/marshmallow-code/marshmallow/wiki/Ecosystem",
+    },
 }
 
 html_sidebars = {
-    "index": ["about.html", "donate.html", "useful-links.html", "searchbox.html"],
+    "index": ["about.html", "searchbox.html", "donate.html", "useful-links.html"],
     "**": [
         "about.html",
+        "searchbox.html",
         "donate.html",
         "useful-links.html",
         "localtoc.html",
         "relations.html",
-        "searchbox.html",
     ],
 }
 
@@ -97,11 +99,6 @@ versionwarning_messages = {
         "This document is for the development version. "
         'For the stable version documentation, see <a href="/en/stable/">here</a>.'
     ),
-    "stable": (
-        "<strong>Black Lives Matter.</strong> "
-        'Support the <a href="https://eji.org/">Equal Justice Initiative</a> and '
-        '<a href="https://colorofchange.org/">Color of Change</a>.'
-    ),
     "2.x-line": (
         "marshmallow 2 is no longer supported as of 2020-08-18. "
         '<a href="https://marshmallow.readthedocs.io/en/latest/upgrading.html#upgrading-to-3-0">'
@@ -112,9 +109,4 @@ versionwarning_messages = {
 versionwarning_body_selector = "div.document"
 versionwarning_banner_title = ""
 # For debugging locally
-# versionwarning_project_version = "stable"
-
-
-def setup(app):
-    # https://docs.readthedocs.io/en/latest/guides/adding-custom-css.html
-    app.add_stylesheet("css/versionwarning.css")
+# versionwarning_project_version = "latest"
