@@ -5,12 +5,13 @@ import itertools
 import decimal
 import uuid
 import ipaddress
+import math
 
 import pytest
 
 from marshmallow import Schema, fields, missing as missing_
 
-from tests.base import User, ALL_FIELDS, central
+from tests.base import User, ALL_FIELDS, central, GenderEnum, HairColorEnum, DateEnum
 
 
 class DateTimeList:
@@ -125,14 +126,14 @@ class TestFieldSerialization:
 
     def test_integer_field_default(self, user):
         user.age = None
-        field = fields.Integer(default=0)
+        field = fields.Integer(dump_default=0)
         assert field.serialize("age", user) is None
         # missing
         assert field.serialize("age", {}) == 0
 
     def test_integer_field_default_set_to_none(self, user):
         user.age = None
-        field = fields.Integer(default=None)
+        field = fields.Integer(dump_default=None)
         assert field.serialize("age", user) is None
 
     def test_uuid_field(self, user):
@@ -145,7 +146,6 @@ class TestFieldSerialization:
         assert field.serialize("uuid2", user) is None
 
     def test_ip_address_field(self, user):
-
         ipv4_string = "192.168.0.1"
         ipv6_string = "ffff::ffff"
         ipv6_exploded_string = ipaddress.ip_address("ffff::ffff").exploded
@@ -166,7 +166,6 @@ class TestFieldSerialization:
         assert field_exploded.serialize("ipv6", user) == ipv6_exploded_string
 
     def test_ipv4_address_field(self, user):
-
         ipv4_string = "192.168.0.1"
 
         user.ipv4 = ipaddress.ip_address(ipv4_string)
@@ -178,7 +177,6 @@ class TestFieldSerialization:
         assert field.serialize("empty_ip", user) is None
 
     def test_ipv6_address_field(self, user):
-
         ipv6_string = "ffff::ffff"
         ipv6_exploded_string = ipaddress.ip_address("ffff::ffff").exploded
 
@@ -195,7 +193,6 @@ class TestFieldSerialization:
         assert field_exploded.serialize("ipv6", user) == ipv6_exploded_string
 
     def test_ip_interface_field(self, user):
-
         ipv4interface_string = "192.168.0.1/24"
         ipv6interface_string = "ffff::ffff/128"
         ipv6interface_exploded_string = ipaddress.ip_interface(
@@ -221,7 +218,6 @@ class TestFieldSerialization:
         )
 
     def test_ipv4_interface_field(self, user):
-
         ipv4interface_string = "192.168.0.1/24"
 
         user.ipv4interface = ipaddress.ip_interface(ipv4interface_string)
@@ -233,7 +229,6 @@ class TestFieldSerialization:
         assert field.serialize("empty_ipinterface", user) is None
 
     def test_ipv6_interface_field(self, user):
-
         ipv6interface_string = "ffff::ffff/128"
         ipv6interface_exploded_string = ipaddress.ip_interface(
             "ffff::ffff/128"
@@ -254,6 +249,31 @@ class TestFieldSerialization:
             == ipv6interface_exploded_string
         )
 
+    def test_enum_field_by_symbol_serialization(self, user):
+        user.sex = GenderEnum.male
+        field = fields.Enum(GenderEnum)
+        assert field.serialize("sex", user) == "male"
+
+    def test_enum_field_by_value_true_serialization(self, user):
+        user.hair_color = HairColorEnum.black
+        field = fields.Enum(HairColorEnum, by_value=True)
+        assert field.serialize("hair_color", user) == "black hair"
+        user.sex = GenderEnum.male
+        field = fields.Enum(GenderEnum, by_value=True)
+        assert field.serialize("sex", user) == 1
+        user.some_date = DateEnum.date_1
+
+    def test_enum_field_by_value_field_serialization(self, user):
+        user.hair_color = HairColorEnum.black
+        field = fields.Enum(HairColorEnum, by_value=fields.String)
+        assert field.serialize("hair_color", user) == "black hair"
+        user.sex = GenderEnum.male
+        field = fields.Enum(GenderEnum, by_value=fields.Integer)
+        assert field.serialize("sex", user) == 1
+        user.some_date = DateEnum.date_1
+        field = fields.Enum(DateEnum, by_value=fields.Date(format="%d/%m/%Y"))
+        assert field.serialize("some_date", user) == "29/02/2004"
+
     def test_decimal_field(self, user):
         user.m1 = 12
         user.m2 = "12.355"
@@ -517,7 +537,7 @@ class TestFieldSerialization:
             name = fields.Field(data_key="")
 
         schema = MySchema()
-        schema.dump({"name": "Grace"}) == {"": "Grace"}
+        assert schema.dump({"name": "Grace"}) == {"": "Grace"}
 
     def test_serialize_with_attribute_and_data_key_uses_data_key(self):
         class ConfusedDumpToAndAttributeSerializer(Schema):
@@ -553,6 +573,38 @@ class TestFieldSerialization:
         field = fields.DateTime(format=fmt)
         assert field.serialize("d", {"d": value}) == expected
 
+    @pytest.mark.parametrize(
+        ("fmt", "value", "expected"),
+        [
+            ("timestamp", dt.datetime(1970, 1, 1), 0),
+            ("timestamp", dt.datetime(2013, 11, 10, 0, 23, 45), 1384043025),
+            (
+                "timestamp",
+                dt.datetime(2013, 11, 10, 0, 23, 45, tzinfo=dt.timezone.utc),
+                1384043025,
+            ),
+            (
+                "timestamp",
+                central.localize(dt.datetime(2013, 11, 10, 0, 23, 45), is_dst=False),
+                1384064625,
+            ),
+            ("timestamp_ms", dt.datetime(2013, 11, 10, 0, 23, 45), 1384043025000),
+            (
+                "timestamp_ms",
+                dt.datetime(2013, 11, 10, 0, 23, 45, tzinfo=dt.timezone.utc),
+                1384043025000,
+            ),
+            (
+                "timestamp_ms",
+                central.localize(dt.datetime(2013, 11, 10, 0, 23, 45), is_dst=False),
+                1384064625000,
+            ),
+        ],
+    )
+    def test_datetime_field_timestamp(self, fmt, value, expected):
+        field = fields.DateTime(format=fmt)
+        assert field.serialize("d", {"d": value}) == expected
+
     @pytest.mark.parametrize("fmt", ["iso", "iso8601", None])
     @pytest.mark.parametrize(
         ("value", "expected"),
@@ -594,7 +646,7 @@ class TestFieldSerialization:
         assert field.serialize("name", user) is None
 
     def test_string_field_default_to_empty_string(self, user):
-        field = fields.String(default="")
+        field = fields.String(dump_default="")
         assert field.serialize("notfound", {}) == ""
 
     def test_time_field(self, user):
@@ -707,16 +759,77 @@ class TestFieldSerialization:
         field = fields.TimeDelta(fields.TimeDelta.MILLISECONDS)
         assert field.serialize("d6", user) == d6_seconds * 1000 + 1
         field = fields.TimeDelta(fields.TimeDelta.MICROSECONDS)
-        assert field.serialize("d6", user) == d6_seconds * 10 ** 6 + 1000 + 1
+        assert field.serialize("d6", user) == d6_seconds * 10**6 + 1000 + 1
 
         user.d7 = None
         assert field.serialize("d7", user) is None
 
+        # https://github.com/marshmallow-code/marshmallow/issues/1856
+        user.d8 = dt.timedelta(milliseconds=345)
+        field = fields.TimeDelta(fields.TimeDelta.MILLISECONDS)
+        assert field.serialize("d8", user) == 345
+
+        user.d9 = dt.timedelta(milliseconds=1999)
+        field = fields.TimeDelta(fields.TimeDelta.SECONDS)
+        assert field.serialize("d9", user) == 1
+
+        user.d10 = dt.timedelta(
+            weeks=1,
+            days=6,
+            hours=2,
+            minutes=5,
+            seconds=51,
+            milliseconds=10,
+            microseconds=742,
+        )
+
+        field = fields.TimeDelta(fields.TimeDelta.MICROSECONDS, float)
+        unit_value = dt.timedelta(microseconds=1).total_seconds()
+        assert math.isclose(
+            field.serialize("d10", user), user.d10.total_seconds() / unit_value
+        )
+
+        field = fields.TimeDelta(fields.TimeDelta.MILLISECONDS, float)
+        unit_value = dt.timedelta(milliseconds=1).total_seconds()
+        assert math.isclose(
+            field.serialize("d10", user), user.d10.total_seconds() / unit_value
+        )
+
+        field = fields.TimeDelta(fields.TimeDelta.SECONDS, float)
+        assert math.isclose(field.serialize("d10", user), user.d10.total_seconds())
+
+        field = fields.TimeDelta(fields.TimeDelta.MINUTES, float)
+        unit_value = dt.timedelta(minutes=1).total_seconds()
+        assert math.isclose(
+            field.serialize("d10", user), user.d10.total_seconds() / unit_value
+        )
+
+        field = fields.TimeDelta(fields.TimeDelta.HOURS, float)
+        unit_value = dt.timedelta(hours=1).total_seconds()
+        assert math.isclose(
+            field.serialize("d10", user), user.d10.total_seconds() / unit_value
+        )
+
+        field = fields.TimeDelta(fields.TimeDelta.DAYS, float)
+        unit_value = dt.timedelta(days=1).total_seconds()
+        assert math.isclose(
+            field.serialize("d10", user), user.d10.total_seconds() / unit_value
+        )
+
+        field = fields.TimeDelta(fields.TimeDelta.WEEKS, float)
+        unit_value = dt.timedelta(weeks=1).total_seconds()
+        assert math.isclose(
+            field.serialize("d10", user), user.d10.total_seconds() / unit_value
+        )
+
+        with pytest.raises(ValueError):
+            fields.TimeDelta(fields.TimeDelta.SECONDS, str)
+
     def test_datetime_list_field(self):
         obj = DateTimeList([dt.datetime.utcnow(), dt.datetime.now()])
         field = fields.List(fields.DateTime)
         result = field.serialize("dtimes", obj)
-        assert all(type(each) == str for each in result)
+        assert all(type(each) is str for each in result)
 
     def test_list_field_serialize_none_returns_none(self):
         obj = DateTimeList(None)
@@ -799,8 +912,8 @@ class TestFieldSerialization:
         obj = DateTimeIntegerTuple((dt.datetime.utcnow(), 42))
         field = fields.Tuple([fields.DateTime, fields.Integer])
         result = field.serialize("dtime_int", obj)
-        assert type(result[0]) == str
-        assert type(result[1]) == int
+        assert type(result[0]) is str
+        assert type(result[1]) is int
 
     def test_tuple_field_serialize_none_returns_none(self):
         obj = DateTimeIntegerTuple(None)
@@ -857,7 +970,7 @@ class TestSchemaSerialization:
     def test_serialize_with_missing_param_value(self):
         class AliasingUserSerializer(Schema):
             name = fields.String()
-            birthdate = fields.DateTime(default=dt.datetime(2017, 9, 29))
+            birthdate = fields.DateTime(dump_default=dt.datetime(2017, 9, 29))
 
         data = {"name": "Mick"}
         result = AliasingUserSerializer().dump(data)
@@ -867,7 +980,7 @@ class TestSchemaSerialization:
     def test_serialize_with_missing_param_callable(self):
         class AliasingUserSerializer(Schema):
             name = fields.String()
-            birthdate = fields.DateTime(default=lambda: dt.datetime(2017, 9, 29))
+            birthdate = fields.DateTime(dump_default=lambda: dt.datetime(2017, 9, 29))
 
         data = {"name": "Mick"}
         result = AliasingUserSerializer().dump(data)
